@@ -1,3 +1,5 @@
+=encoding utf8
+
 =head1 NAME
 
 SQL::Steno - Short hand for SQL and compact output
@@ -63,9 +65,9 @@ with other DBs.
 
 =cut
 
-use v5.12;
+use v5.16;
 
-package SQL::Steno v0.2.4.1;
+package SQL::Steno v0.2.99.0;
 
 use strict;
 use warnings;
@@ -133,45 +135,48 @@ for my $fmt ( keys %render ) {
     $Queries_help{".$fmt"} = "   output '&.$fmt() this' or next query as \U$fmt";
     $Queries{".$fmt"} = \&set_render;
 }
-sub Queries(@) {
-    for( @_ ) {
-	$Queries_help{$_->[0]} = $_->[1];
-	$Queries{$_->[0]} = $_->[2];
-    }
+sub Query {
+    $Queries_help{$_[0]} = $_[1];
+    $Queries{$_[0]} = $_[2];
+    undef;
 }
-Queries
+Query @$_ for
     ['-' => "   output next query as YAML",
       '&.yaml'],
 
 
     [ps => '   show processlist (without Sleep)',
-      '{($_[7] // "") ne "Sleep"}=show processlist'],
+      '{(${_}[7] // "") ne "Sleep"}=show processlist'],
 
     [psf => '   show full processlist (without Sleep)',
-      '{($_[7] // "") ne "Sleep"}=show full processlist'],
+      '{(${_}[7] // "") ne "Sleep"}=show full processlist'],
+
+
+    [t => 'unquotedtablename[,unquotedcolumnpattern]   show fields',
+      'COLUMN_NAME,COLUMN_TYPE,IS_NULLABLE NUL,COLUMN_KEY `KEY`,COLUMN_COMMENT;information_schema.`COLUMNS`;TABLE_SCHEMA=schema() and TABLE_NAME=$\1$? and COLUMN_NAME like$\*??:ob ORDINAL_POSITION'],
 
 
     [s => 'var;value   set @var = value',
       'set @$1=$2'],
 
-    [ss => 'var;value   set @var = "value"',
-      'set @$1=$"*"'],
+    [ss => "var;value   set \@var = 'value'",
+      'set @$1=$\2'],
 
     [sd => 'var;value   set @var = cast("value" as date)',
-      'set @$1=cast($"*" as date)'],
+      'set @$1=cast($\2 as date)'],
 
     [sdt => 'var;value   set @var = cast("value" as datetime)',
-      'set @$1=cast($"*" as datetime)'],
+      'set @$1=cast($\2 as datetime)'],
 
     [st => 'var;value   set @var = cast("value" as time)',
-      'set @$1=cast($"*" as time)'],
+      'set @$1=cast($\2 as time)'],
 
     [sy => '            set @a, @z   yesterday is between @a and @z  (see :baz)',
       #'set @a=date(now())-interval 1 day, @z=date(now())-interval 1 second',
      'select @a:=date(now()-interval 1 day)`@a`, @z:=date(now())-interval 1 second`@z`'];
 
 
-our $weekstart = 1;
+our $weekstart = 1;		# Monday is the first day of the week as per ISO 8601.
 my $timespec_re = qr/[yqmwdhMs]?/;
 our %Join_clause;
 our %Macros =
@@ -352,25 +357,83 @@ sub find($$$\%;\@) {
     $error = 1;
 }
 
-sub convert_Queries($$) {
-    my $res = find $_[0], '&', '', %Queries;
-    local $_ =  $_[1];
-    &convert_table_column;
-    my @arg = split ';';
-    return &$res( $_[0], @arg ) // '' if ref $res;
-    my @rest;
-    for my $i ( 1..@arg ) {
-	$res =~ s/\$$i/$arg[$i - 1]/g or $res =~ s/\$"$i"/"$arg[$i - 1]"/g or push @rest, $arg[$i - 1];
+my %rq = ('[', ']',
+	  '{', '}');
+my $quote_re = qr(\\([-,:;./ #?\N{GREEK SMALL LETTER OMEGA}^\\\@!'"`[\]{}]*)(?:%(.+?))?);
+sub quote($$@) {
+    (local $_, my $join) = splice @_, 0, 2;
+    $join //= ',';
+    return join $join, @_ unless defined;
+    my @args = @_;
+    /(['"`[{])/;
+    my( $lq ) = /(['"`[{])/;
+    $lq ||= "'";
+    my $rq = $rq{$lq} || $lq;
+    my( $noquote, $number, $boolean, $null, $space, $var ) =
+	(tr/!//, tr/#//, tr/?//, tr/\N{GREEK SMALL LETTER OMEGA}^//, tr/\\//, tr/@//);
+    my $split = tr/-// ? '-' : ''; # avoid range by putting - 1st
+    $split .= tr/ // ? '\s' : '';  # space means any whitespace
+    $split .= join '', /([,:;.\/])/g;
+    $split ||= ',';
+    $split = $space ? qr/[$split]/ : qr/\s*[$split]\s*/;
+    join $join, map {
+	if( $noquote || $boolean && /^(?:true|false)$/i || $null && /^null$/i || $var && /^\@\w+$/ ) {
+	    $_;
+	} elsif( $number && /^[-+]?(?:0b[01]+|0x[\da-f]+|(?=\.?\d)\d*\.?\d*(?:e[-+]?\d+)?)$/i ) {
+	    $_;
+	} else {
+	    s/$rq/$rq$rq/g;
+	    "$lq$_$rq";
+	}
+    } map {
+	unless( $space ) {
+	    s/\A\s*//;
+	    s/\s*\Z//;
+	}
+	split $split, $_, -1;
+    } @args;
+}
+
+sub convert_Query($$) {
+    my $name = $_[0];
+    my $res = find $name, '&', '', %Queries;
+    my $ref = ref $res;
+    my @arg;
+    for( $ref ? $_[1] : "$res\cA$_[1]" ) {
+	&convert_table_column;
+	($res, $_) = split "\cA" unless $ref;
+	@arg = split ';';
     }
-    $res =~ s!\$\?(.*?)\?(.*?)\?!@rest ? $1 : $2!e;
-    $res =~ s!\$"\*"!'"' . join( '","', @rest ) . '"'!ge or
-	$res =~ s!\$\*!join( ',', @rest )!ge;
+    return &$res( $name, @arg ) // '' if $ref;
+
+    my( @var, %seen, @rest );
+    $res =~ s(\$$quote_re?(?:(\d+\b)|([*>_]))) {
+	if( $3 && $3 > @arg ) {
+	    '';
+	} else {
+	    push @var, [$1, $2, $4 ? (undef, $4) : $3-1];
+	    undef $seen{$3-1} if $3;	# make it exist
+	    "\cV$#var\cZ";
+	}
+    }eg;
+    if( @arg > keys %seen ) {
+	@rest = @arg;
+	undef $rest[$_] for keys %seen;
+	@rest = grep defined(), @rest;
+    }
+    $res =~ s(\cV(\d+)\cZ) {
+	my @res = @{$var[$1]};
+	quote $res[0], $res[1],
+	    $res[3] ? ($res[3] eq '*' ? @arg :
+		       $res[3] eq '>' ? @rest :
+		       $_) :
+	    $res[2] < 0 ? $name : $arg[$res[2]];
+    }eg;
     $res;
 }
 
 my @keys_Table_Columns;
 sub convert_table_column {
-    # \todo .col. or .col.alias like #tbl#
     @keys_Table_Columns = keys %Table_Columns unless @keys_Table_Columns;
     s&(?<!\\)#(\w+)(?:#(\w*))?&find( $1, '#', '', %Tables, @keys_Table_Columns ) . ($2 ? " $2" : defined $2 ? " $1" : '')&eg unless $error;
 
@@ -384,7 +447,7 @@ sub convert_table_column {
 	    undef $column{$_} for @{$Table_Columns{$_}};
 	}
 	my @column = keys %column;
-	s/(^|[-+\s(,;&|])?(?<!\\)\.([a-z]\w*)(?:\.(\w*))?/(defined $1 ? $1 : '.') . (find $2, '.', '', %Columns, @column) . ($3 ? " $3" : defined $3 ? " $2" : '')/egi;
+	s/(^|.&|[-+\s(,;|])?(?<!\\)\.([a-z]\w*)(?:\.(\w*))?/(defined $1 ? $1 : '.') . (find $2, '.', '', %Columns, @column) . ($3 ? " $3" : defined $3 ? " $2" : '')/egi;
     }
 }
 
@@ -394,15 +457,112 @@ sub convert_table_column {
 This function takes a short-hand query in C<$_> and transforms it to SQL.  See
 L</shell> for more run time oriented features.
 
-First it looks for C<:macro>.  These are mostly simple text-replacements
-stored in C<%Macros>.  There are also some dynamic macros.  Those starting
-with C<:j> (join) or C<:lj> (left join) may continue into a table spec without
-the leading C<#>.  E.g. C<:ljtbl#t> might expand to C<left join table t>.
+=head3 C<:\I<spec>(I<strings>)> E<nbsp> or E<nbsp> C<:\I<spec>%I<join>(I<strings>)>
+
+This is a special macro that transforms odd lists to SQL syntax.  It takes a
+list of unquoted strings and quotes each one of them in various ways.  The
+syntax is inspired by the Shell single character quote and Perl's C<\(...)>
+syntax.  The I<strings> can get split on a variety of one or even
+simultaneously various characters, which you can give in any order in the
+I<spec>:
+
+=over
+
+=item C<\> (backslash)
+
+This one isn't a character to split on, but rather prevents trimming the
+whitespace of both ends of the resulting strings.
+
+=item C<,> (comma), the default
+
+You only need to specify it, if you want to split on commas, as well as other
+characters, at the same time.
+
+=item C<E<nbsp>> (space)
+
+This one stands for any single whitespace.  Since strings are normally
+trimmed, it's the equivalent of what the Shell does.  But, if you combine it
+with C<\>, which prevents trimming, you will get an empty string between each
+of multiple whitespaces.
+
+=item C<-> (minus)
+
+=item C<:> (colon)
+
+=item C<;> (semicolon)
+
+=item C<.> (period)
+
+=item C</> (slash)
+
+=back
+
+The I<spec> can also contain several of these characters to prevent certain
+strings from being quoted:
+
+=over
+
+=item C<#> (hash)
+
+All numbers, including signed, floats, binary and hexadecimal stay literal.
+If you use C<-> as a separator, there can be no negative numbers.
+
+=item C<?> (question mark)
+
+The boolean values C<true> and C<false> stay literal.
+
+=item C<ω> (omega) E<nbsp> or E<nbsp> C<^> (caret)
+
+The value C<null> stays literal.
+
+=item C<@> (at sign)
+
+Variables C<@name> stay literal.
+
+=item C<!> (exclamation mark)
+
+Everything, presumably column names, stays literal.
+
+=back
+
+And you can give at most one I<spec> for the quotes to use:
+
+=over
+
+=item C<'> (quote), the default
+
+=item C<"> (double quote)
+
+=item C<`> (backquote)
+
+=item C<[]> (brackets)
+
+=item C<{}> (braces)
+
+In the latter two cases, the closing quotes are optional, for decoration only,
+or if code completion adds them.
+
+=back
+
+The results are joined by comma, unless I<join> is given.  E.g. C<:\(a,b,
+c,NULL,true,-1.2)> gives C<'a','b','c','NULL','true','-1.2'>, while
+C<:\:"/\#?0(a:b/ c:NULL:true/-1.2)> gives C<"a","b"," c",NULL,true,-1.2> and
+C<:\ !%&&(a b E<nbsp>c)> gives C<a&&b&&c>.
+
+
+
+=head3 C<:I<macro>>
+
+These are mostly simple text-replacements stored in C<%Macros>.  Unlike
+L<C<:\>|/spec-strings-or-:-spec-join-strings> these do not take arguments.
+There are also some dynamic macros.  Those starting with C<:j> (join) or
+C<:lj> (left join) may continue into a L<table spec|/tbl-or-tbl-or-tbl-alias>
+without the leading C<#>.  E.g. C<:ljtbl#t> might expand to C<left join table
+t>.
 
 Those starting with C<:gb> (group by) or C<:ob> (order by) may be followed by
 result columns numbers from 1-9, each optionally followed by a or d for asc or
-desc.  Macros can also be C<:{Perl code}>, which gets replaced by what it
-returns.
+desc.  E.g. C<:ob2d3> gives C<order by 2 desc, 3>.
 
 Then there are the time macros, where an optional leading letter indicates a
 base time, and an optional trailing letter with an optional count means the
@@ -444,28 +604,42 @@ offset.  The letters are:
 
 =back
 
-Then it looks for C<#tbl>, C<#tbl#> or C<#tbl#alias>.  Here tbl is a key of
-C<%Tables> or any abbreviation of known tables in C<@Tables>.  If followed by
-C<#>, the abbreviation is used as an alias, unless an alias directly follows,
-in which case that is used.
 
-Then it looks for C<.col>, C<.col.> C<.col.alias>.  Here col is a key of
-C<%Columns> or any abbreviation of columns of any table recognized in the
-query.  If followed by C<.>, the abbreviation is used as an alias, unless an
-alias directly follows, in which case that is used.  It tries to be clever
-about whether the 1st C<.> needs to be preserved, i.e. following a table name.
+=head3 C<:{I<Perl code>}>
 
-Finally it looks for C<func(>.  Here func is a key of C<%Functions> or any
+This gets replaced by what it returns.
+
+
+=head3 C<#I<tbl>> E<nbsp> or E<nbsp> C<#I<tbl>#> E<nbsp> or E<nbsp> C<#I<tbl>#I<alias>>
+
+Here I<tbl> is a key of C<%Tables> or any abbreviation of known tables in
+C<@Tables>.  If followed by C<#>, the abbreviation is used as an alias, unless
+an I<alias> directly follows, in which case that is used.
+
+
+=head3 C<.I<col>> E<nbsp> or E<nbsp> C<.I<col>.> E<nbsp> or E<nbsp> C<.I<col>.I<alias>>
+
+Here I<col> is a key of C<%Columns> or any abbreviation of columns of any table
+recognized in the query.  If followed by C<.>, the abbreviation is used as an
+alias, unless an I<alias> directly follows, in which case that is used.  It tries
+to be clever about whether the 1st C<.> needs to be preserved, i.e. following
+a table name.
+
+=head3 C<I<func>(> E<nbsp> or E<nbsp> C<I<func>\I<spec>(I<strings>)> E<nbsp> or E<nbsp> C<I<func>\I<spec>%I<join>(I<strings>)>
+
+Here I<func> is a key of C<%Functions> or any
 abbreviation of known functions in C<@Functions>, which includes words
 typically followed by an opening parenthesis, such as C<u(> for C<using(>.
-C<i(> is C<in(>, so that C<in(> is free for matching C<ifnull(>, but more
-importantly, because it has a smart brother: C<I(> is also C<in(>, where
-quoting is applied as necessary, and unless you give an initial comma C<I(,>,
-the elements are separated on space.  E.g. C<I(a 3 a's q"q)> and
-C<I(,a,3,a's,q"q)> both give C<in("a",3,"a's",'q"q')>.  If the 1st argument of
-a function is empty and the abbrev or function is found in
-C<%DefaultArguments> the value becomes the 1st argument.
+C<i(> becomes C<in(>, whereas C<in(> becomes C<ifnull(>.
 
+If the 2nd or 3rd form is used, the I<strings> inside of the parentheses are treated
+just like C<L<:\I<spec>(I<strings>)|/spec-strings-or-:-spec-join-strings>>,
+but in this case preserving the parentheses.
+
+If the 1st argument of a function is empty and the abbrev or function is found
+in C<%DefaultArguments> the value becomes the 1st argument.
+
+=head3 Abbreviated Keyword
 
 Finally it picks on the structure of the statement: These keywords can be
 abbreviated: C<se(lect)>, C<ins(ert)>, C<upd(ate)> or C<del(ete)>.  If none of
@@ -480,19 +654,23 @@ are frst replaced by C<set> and then C<where>.
 =cut
 
 sub convert {
-    # Handle I(str1 str2 str3)
-    s<\bI\((, ?)?(.+?)\)> { 'i('.join(',', map { /"/ ? "'$_'" : /^\d+$/ ? $_ : qq+"$_"+ } split $1||' +', $2, -1).')' }ge;
-
+    # Handle :\,,,(str1, str2, str3)
+    s<(?:\b(\w+)|:)$quote_re\((.+?)\)> {
+	($1 ? "$1(" : '') .
+ 	quote($2, $3, $4 ) .
+	($1 ? ')' : '')
+    }ge;
     my @strings;		# extract strings to prevent following replacements inside.
-    while( /\G.*?(['"])/gc ) {
-	my $quote = $1;
+    while( /\G.*?(['"`[{])/gc ) {
+	my $rq = $rq{$1}||$1;
 	my $pos = pos;
-	while( /\G.*?([\\$quote])/gc ) {
+	while( /\G.*?([$rq\\])/gc ) {
 	    if( $1 eq '\\' ) {
 		++pos;		# skip next
-	    } elsif( ! /\G$quote/gc ) { # skip double quote
-		push @strings, substr $_, $pos - 1, 1 - $pos + pos, # get string
-		    "\cA".@strings."\cB"; # and replace with counter
+	    } elsif( ! /\G$rq/gc ) { # skip doubled quote
+		push @strings,
+		    substr $_, $pos - 1, 1 - $pos + pos, # get string
+		    "\cS".@strings."\cZ"; # and replace with counter
 		last;
 	    }
 	}
@@ -526,7 +704,7 @@ sub convert {
 
     s/ $//mg;
     s/ {2,}/ /g;
-    s/\cA(\d+)\cB/$strings[$1]/g; # put back the strings
+    s/\cS(\d+)\cZ/$strings[$1]/g; # put back the strings
 
     1;
 }
@@ -596,6 +774,8 @@ sub render_csv($;$$) {
     }
 }
 
+our $NULL = 'ω';
+utf8::decode $NULL;
 my( $r1, $r2, $r3, $r5 ) = ('[01]\d', '[0-2]\d', '[0-3]\d', '[0-5]\d');
 sub render_table($;$$) {
     my( $sth, $filter ) = @_;
@@ -607,12 +787,12 @@ sub render_table($;$$) {
 	 exists $_[2]{date},
 	 exists $_[2]{time})
 	if $_[2];
-    $null ||= '\@';
+    $null ||= $NULL;
     my @name = @{$sth->{NAME}};
     my @len = (0) x @name;
     my( @txt, @res, @comp );
     while( my @res1 = $sth->fetchrow_array ) {
-	next if $filter && !$filter->( \@name, @res1 );
+	$filter->( \@name, @res1 ) or next if $filter;
 	for my $i ( 0..$#res1 ) {
 	    if( !defined $res1[$i] ) {
 		$res1[$i] = $null;
@@ -785,39 +965,44 @@ continuation lines as long as you put a backslash at the end of your lines.
 Or there is a special case, if the 1st line starts with C<\\>, then everything
 up to C<\\> at the end of one of the next lines, constitutes one entry.
 
-In addition to converting it offers a few extra features, performed in this
-order (i.e. C<&xyz> can return C</regexp/=literal sql> etc.):
+In addition to converting, it offers a few extra features, performed in this
+order (i.e. C<&I<xyz>> can return C</I<regexp>/=I<literal sql>> etc.):
 
-=over
+=head3 C<&{I<Perl code>} I<following text>>
 
-=item &query arg; ...
+Run I<Perl code>.  It sees the optional I<following text> in C<$_> and may
+modify it.  If it returns undef this statement is skipped.  If it returns a
+DBI statement handle, run that instead of this statement.  Else replace with
+what it returns.
 
-=item &query( arg; ... ) following text
+Reprocess result as a shell entry (i.e. it may return another C<&I<query>>).
 
-=item &{Perl code} following text
+=head3 C<&I<query arg>; ...> E<nbsp> or E<nbsp> C<&I<query>( I<arg>; ... ) I<following text>>
 
 These allow canned entries and are more complex than macros, in that they take
 arguments and replacement can depend on the argument.
 
-Run I<Perl code>.  It sees the I<following text> in C<$_> and may modify it.
-If it returns undef this statement is skipped.  If it returns a DBI statement
-handle run that instead of this statement.  Else replace with what it returns.
+Reprocess result as a shell entry (i.e. it may return another C<&I<query>>).
 
-Reprocess result as a shell entry (i.e. it may return another C<&>query).
+You can define your own canned queries with:
 
-=item /regexp/ statement
+C<    &{ Query I<name> =E<gt> 'I<doc>', 'I<query>' }>
 
-=item /regexp/i statement
+Here C<query> becomes the replacement string for C<&name>.  It may contain
+arguments a bit like the Shell: C<$0> (I<name>), C<$*> (all arguments), C<$1,
+$2, ..., $10, ...> (individual arguments) and C<$E<gt>> (all arguments not
+adressed individually).  They can become quoted like
+L<C<:\>|/spec-strings-or-:-spec-join-strings> as C<:\I<spec>I<arg>> or
+C<:\I<spec>%I<join>I<arg>>: E.g.: C<$\-"1> splits the 1st (semi-colon
+separated from the 2nd) argument itself on C<-> (minus), quotes the pieces
+with C<"> (double quote) and joins them with C<,> (comma).  Putting the quotes
+inside the argument like this, eliminates them, if no argument is given.
 
-=item !/regexp/ statement
+=head3 C</I<regexp>/ I<statement>> E<nbsp> or E<nbsp> C</I<regexp>/i I<statement>> E<nbsp> or E<nbsp> C<!/I<regexp>/ I<statement>> E<nbsp> or E<nbsp> C<!/I<regexp>/i I<statement>>
 
-=item !/regexp/i statement
-
-This will treat the I<statement> normally, but will join each output row for
-matching as though it were the only row in default table format.  That is
-there is no space padding and the character '|' (which must be matched as
-C<\|>) separates columns.  At this stage no value rewriting has occured,
-except NULL is already rendered as C<\@>.
+This will treat the I<statement> normally, but will join each output row into
+a C<~> (tilde) separated string for matching.  NULL fields are rendered as that
+string.
 
 With a suffix C<i>, matching becomes case insensitive.  This is why the mostly
 optional space is shown above.  Without an C<i>, but if the statement starts
@@ -828,7 +1013,17 @@ you must separate it with a space.
 Only matching rows are considered unless there is a preceding C<!>, in which
 case only non-matching rows are considered.
 
-=item {Perl code}statement
+You can provide your own formatting of the row by setting C<$regexp_fail> to a
+perl sub that returns a Perl expression as a string.  That expression takes
+the row in C<@_> and shall be true if the row fails to match.
+
+Caveat: the whole result set of the I<statement> gets generated and
+transferred to the client.  This is definitely much more expensive than doing
+the equivalent filtering in the where clause.  But it is not a big deal for
+tens or maybe hundreds of thousands or rows, probably still faster than
+writing the corresponding SQL.  And Perl's regexps are so much more powerful.
+
+=head3 C<{I<Perl code>}I<statement>>
 
 Call I<Perl code> for every output row returned by the I<statement> with the
 array of column names as zeroth argument and the values after that (i.e.
@@ -838,42 +1033,40 @@ false, the row is skipped.
 You may combine S<C</regexp/{Perl code}>> in any order and as many of them as
 you want.
 
-=item =literal sql
+The same caveat as for regexps applies here.  But again Perl is far more
+powerful than any SQL functions.
+
+=head3 C<=I<literal sql>>
 
 A preceding C<=> prevents conversion, useful for hitherto untreated keywords
 or where the conversion doesn't play well with your intention.
 
-=item ?
+=head3 C<?>
 
 Help prefix.  Alone it will give an overview.  You can follow up with any of
 the special syntaxes, with or without an abbreviation.  E.g. C<?(> will show
 all function abbreviations, whereas C<?abbrev(> will show only those functions
 matching abbrev or C<?#abbrev> only those tables matching abbrev.
 
-=item ??statemment
+=head3 C<??I<statement>>
 
-Will convert and show, but not perform statement.  If C<$dbh> is not set, this
+Will convert and show, but not perform I<statement>.  If C<$dbh> is not set, this
 is the default behaviour.
 
-=item !System Shell code
+=head3 C<!I<System Shell code>>
 
 Run it.
 
-=item >filename
-
-=item >>filename
+=head3 C<E<gt>I<filename>> E<nbsp> or E<nbsp> C<E<gt>E<gt>I<filename>>
 
 Redirect or append next statement's output to I<filename>.  For known
-suffixes and options, see the L<next section|/Output formats>.
+suffixes and options, see the L<next section|/Output Formats>.
 
-=item |System Shell code
+=head3 C<|I<System Shell code>>
 
 Pipe next statement's output through I<System Shell code>.
 
-=back
-
-
-=head2 Output formats
+=head2 Output Formats
 
 The output format for the next SQL statement that is run, is chosen from the
 suffix of a redirection or a special suffix query.  In both cases
@@ -952,7 +1145,8 @@ hh: or hh:mm(:00) to hh:mm.
 
 =item null
 
-Output this keywork instead of the shorter C<\@>.
+Output this keyword instead of the shorter C<ω> (or whatever you assigned to
+C<$NULL>).
 
 =back
 
@@ -971,6 +1165,9 @@ for it.
 
 our $prompt = 'steno> ';
 our $contprompt = '...>  ';
+our $echo;
+# Called for every leading re, 1st arg is the optional '!', 2nd arg '/re/' or '/re/i'.  Expression shall be true for non-matching lines.
+our $regexp_fail = sub($$) { 'join( "~", map ref() ? () : $_ // q!NULL!, @_ )' . ($_[0] ? '=~' : '!~') . $_[1] };
 sub shell() {
     print STDERR $prompt;
     my $fh;
@@ -990,13 +1187,10 @@ sub shell() {
 	    s/\A\s+//;
 	}
 	s/\s+\Z//;
+	say if $echo;
 	until( $error ) {
-	    last unless
-		s!^\&(\.?\w+|-)(\(((?:(?>[^()]+)|(?2))*)\))!convert_Queries $1, $3!e
-		or s!^\&(\.?\w+|-) *(.*)!convert_Queries $1, $2!e
-		or my $perl = s!^\&$perl_re!!;
-	    if( $perl ) {
-		$perl = eval $1;
+	    if( s!^&$perl_re!! ) {
+		my $perl = eval $1;
 		local $| = 1;	# flush to avoid stderr prompt overtaking last output line.
 		warn $@ if $@;
 		if( UNIVERSAL::isa $perl, 'DBI::st' ) {
@@ -1007,13 +1201,17 @@ sub shell() {
 		} else {
 		    goto NEXT;
 		}
+	    } else {
+		last unless
+		    s!^&(\.?\w+|-)(\(((?:(?>[^()]+)|(?2))*)\))!convert_Query $1, $3!e
+		    or s!^&(\.?\w+|-) *(.*)!convert_Query $1, $2!e;
 	    }
 	}
 
 	my $filter = '';
 	while( s/^\s*$perl_re// || s%^\s*(!?)(/.+?/(?:i\b)?)\s*%% ) {
 	    if( defined $2 ) {
-		$filter .= 'return if join( "|", map ref() ? () : $_ // q!\@!, @_ ) ' . ($1 ? '=' : '!') .  "~ $2;\n";
+		$filter .= 'return if ' . $regexp_fail->( $1, $2 ) .  ";\n";
 	    } else {
 		$filter .= "return unless eval $1;\n";
 	    }
@@ -1126,12 +1324,11 @@ sub helphash($$$\%;\@) {
 
 sub help {
     if( defined $_[2] ) {
-	local $Functions{I} = 'in  but quotes strings itself, splits on space, or with beginning if I(, ... or I(,...';
 	helphash $_[2], '', '(', %Functions, @Functions;
     } elsif( !$_[0] ) {
 	print <<\HELP;
 All entries are single line unless \\wrapped at 1st bol and last eol\\ or continued.\
-Queries have the form: {{i}/regexp/}{=}query
+Queries have the form: {{!}/regexp/{i}}{=}query
 The query has lots of short-hands expanded, unless it is prefixed by the optional =.
 The fields joined with '|' are grepped if regexp is given, case-insensitively if i is given.
 
@@ -1145,8 +1342,8 @@ fieldlist defaults to '*', also if Query starts with '#'.
 ';' is alternately replaced by 'from' and 'where' except '\;'.
 
 Abbreviations, more help with ?&{abbrev}, ?#{abbrev}, ?.{abbrev}, ?{abbrev}(, ?:{abbrev}
-&query $1,$2,...	# only at bol
-&query($1,$2,...)...	# only at bol, only replace upto )
+&query $1;$2;...	# only at bol
+&query($1;$2;...)...	# only at bol, only replace upto )
 &{Perl code}...		# only at bol, if it return undef then skip, else prepend to ...
 #table
 .column			# for any table recognized in statement
@@ -1165,6 +1362,14 @@ HELP
 	$error or print "$_\n" if
 	    $_[1] and $_ = find $_[1], '.', '', %Columns; # \todo, @column;
     } elsif( $_[0] eq '&' ) {
+	print <<\HELP unless $_[1];
+&{ Query name => 'doc', 'query' }   define query &name on the fly
+	query may contain arguments a bit like the Shell: $1, $2, ..., $*
+	they can become quoted: $'1', $"2", $`*`, $[3], ${4}
+	$* means the remaining args after using up the numbered ones
+	if it is quoted, each arg gets quoted, separated by a comma
+	$?args-replacement?no-args-replacement? 1st if remaining args
+HELP
 	helphash $_[1], '&', '', %Queries_help;
     } else {
 	local $Tables{TBL} = 'TABLE';
@@ -1172,6 +1377,8 @@ HELP
     }
 }
 
+1;
+
 =head1 YOUR SCRIPT
 
     package SQL::Steno;		# doesn't export yet, so get the functions easily
@@ -1195,4 +1402,4 @@ L<DBI>, L<SQL::Interp>, L<SQL::Preproc>, L<SQL::Yapp>, L<Jade|http://jade-lang.c
 
 =head1 AUTHOR
 
-(C) 2015 by Daniel Pfeiffer <occitan@esperanto.org>.
+(C) 2015, 2016 by Daniel Pfeiffer <occitan@esperanto.org>.
